using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SetupGrid.Properties;

namespace SetupGrid
{
    #region [ AsyncSetupGrid Control ]
    public partial class AsyncSetupGrid : DataGridView
    {
        public AsyncSetupGrid()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            // TODO: Add custom paint code here

            // Calling the base class OnPaint
            base.OnPaint(pe);
        }
    }
    #endregion

    #region [ DataGridViewProgressColumn ]
    public class DataGridViewProgressCell : DataGridViewImageCell
    {
        #region [ Private Member(s) ]
        private ProgressBar _progressBar; 
        #endregion

        #region [ Constructor(s) ]
        public DataGridViewProgressCell()
        {
            ValueType = typeof(int);
            _progressBar = new ProgressBar();
        } 
        #endregion

        #region [ Overrided Method(s) ]
        protected override object GetFormattedValue(object value, int rowIndex, ref DataGridViewCellStyle cellStyle, TypeConverter valueTypeConverter, TypeConverter formattedValueTypeConverter, DataGridViewDataErrorContexts context)
        {
            if (value != null && value != System.DBNull.Value)
            {
                Bitmap bmptxt = new Bitmap(OwningColumn.Width - 1, OwningRow.Height - 1);
                Label lbl = new Label();
                lbl.AutoSize = false;
                lbl.TextAlign = ContentAlignment.MiddleCenter;
                lbl.Height = bmptxt.Height - 4;
                lbl.Width = bmptxt.Width - 4;
                lbl.BackColor = cellStyle.BackColor;

                if (((int)value) < 0)
                {
                    SetupDS.tblSetupScriptsRow row =
                        ((System.Data.DataRowView)(((System.Windows.Forms.DataGridViewElement)(this)).DataGridView.Rows[rowIndex].DataBoundItem)).Row as SetupDS.tblSetupScriptsRow;
                    lbl.ForeColor = Color.Red;
                    lbl.Text = row.Message;
                    lbl.DrawToBitmap(bmptxt, new System.Drawing.Rectangle(2, 2, bmptxt.Width - 4, bmptxt.Height - 4));
                }
                else
                {
                    Bitmap bmp = new Bitmap(OwningColumn.Width - 1, OwningRow.Height - 1);
                    _progressBar.Height = bmp.Height - 4;
                    _progressBar.Width = bmp.Width - 4;
                    _progressBar.BackColor = cellStyle.BackColor;
                    _progressBar.ForeColor = cellStyle.ForeColor;
                    _progressBar.Style = ProgressBarStyle.Continuous;
                    _progressBar.Value = (int)value;
                    _progressBar.Update();
                    _progressBar.DrawToBitmap(bmp, new System.Drawing.Rectangle(2, 2, bmp.Width - 4, bmp.Height - 4));

                    lbl.Image = bmp;
                    lbl.Text = String.Format("{0}%", (int)value);
                    lbl.DrawToBitmap(bmptxt, new System.Drawing.Rectangle(2, 2, bmp.Width - 4, bmp.Height - 4));
                }
                return bmptxt;
            }
            else
            {
                return null;
            }
        } 
        #endregion
    }

    public class DataGridViewProgressColumn : DataGridViewImageColumn
    {
        #region [ Constructor(s) ]
        public DataGridViewProgressColumn()
        {
            CellTemplate = new DataGridViewProgressCell();
        } 
        #endregion
    } 
    #endregion

    #region [ DataGridViewDoneColumn ]
    public class DataGridViewDoneCell : DataGridViewImageCell
    {
        #region [ Constructor(s) ]
        public DataGridViewDoneCell()
        {
        } 
        #endregion

        #region [ Overrided Method(s) ]
        protected override object GetFormattedValue(object value, int rowIndex, ref DataGridViewCellStyle cellStyle, TypeConverter valueTypeConverter, TypeConverter formattedValueTypeConverter, DataGridViewDataErrorContexts context)
        {
            if (value != null && value != System.DBNull.Value)
            {
                switch ((int)value)
                {
                    case -1:
                        return Resource.Error;
                    case 0:
                        return null;
                    case 100:
                        return Resource.DoneCheck;
                    default:
                        return Resource.Play;
                }
            }
            else
            {
                return null;
            }
        } 
        #endregion
    }

    public class DataGridViewDoneColumn : DataGridViewImageColumn
    {
        #region [ Constructor(s) ]
        public DataGridViewDoneColumn()
        {
            CellTemplate = new DataGridViewDoneCell();
        } 
        #endregion
    }
    #endregion
}
