using System;
using System.Collections.Generic;
using System.Text;

namespace SetupGrid.ScriptCommands
{
    #region [ Progress Event ]
    public class ProgressEventArgs : EventArgs
    {
        private int _percent;

        public int Percent
        {
            get { return _percent; }
            set { _percent = value; }
        }

        public ProgressEventArgs(int percent)
        {
            _percent = percent;
        }
    }
    public delegate void ProgressEventHandler(object sender, ProgressEventArgs e);
    #endregion

    public abstract class BaseCommand : IBaseCommand
    {
        #region [ Event(s) ]
        public event ProgressEventHandler Progress; 
        #endregion

        #region [ Private Field(s) ]
        private SetupGrid.Properties.SetupDS.tblSetupScriptsRow _script;
        private ScriptKind _kind;
        private ISetupGridUC _ctrl;
        #endregion

        #region [ Protected members ]
        protected ISetupGridUC SetupGridUC
        {
            get { return _ctrl; }
        }

        protected void OnProgress(object sender, ProgressEventArgs e)
        {
            if (Progress != null)
            {
                if (e.Percent >= 100)
                    e.Percent = 100;
                Progress(sender, e);
            }
        }
        #endregion

        #region [ public SetupDS.tblSetupScriptsRow Script ]
        public SetupGrid.Properties.SetupDS.tblSetupScriptsRow Script
        {
            get { return _script; }
        }
        #endregion

        #region [ Constructor(s) ]
        private BaseCommand() { }

        public BaseCommand(SetupGrid.Properties.SetupDS.tblSetupScriptsRow script, ISetupGridUC ctrl)
        {
            _script = script;
            _ctrl = ctrl;
        } 
        #endregion

        #region [ Abstract method(s) ]
        public abstract void Execute();

        public abstract ScriptKind Kind
        {
            get;
        } 
        #endregion
    } 

}
