﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Data.SqlClient;
using System.Data;
using System.IO;

namespace SetupGrid.ScriptCommands
{
    public class SQLScriptCommand : BaseCommand
    {
        #region [ Constructor(s) ]
        public SQLScriptCommand(SetupGrid.Properties.SetupDS.tblSetupScriptsRow script, ISetupGridUC ctrl)
            : base(script, ctrl)
        {
        } 
        #endregion

        #region [ Overrided Method(s) ]
        public override void Execute()
        {
            OnProgress(this, new ProgressEventArgs(0));
            if (File.Exists(Script.Script))
            {
                TextReader tr = new StreamReader(Script.Script);
                try
                {
                    ExecuteCommand(tr.ReadToEnd(), true);
                }
                finally
                {
                    tr.Close();
                }
            }
            else
            {
                string strAppDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);

                throw new Exception(this.Kind + ": " + Script.Script + " file does not exist in: " + strAppDir);
            }
        }
        public override ScriptKind Kind
        {
            get { return ScriptKind.SQLScript; }
        }
        #endregion

        #region [ Private Method(s) ]
        private bool ExecuteCommand(string script, bool bTran)
        {
            bool bret = true;
            OnProgress(this, new ProgressEventArgs(0));
            using (SqlConnection con = new SqlConnection())
            {
                con.ConnectionString = SetupGrid.Properties.Settings.Default.DBConnectionString;
                con.Open();

                string[] _delimeter = new string[] { "GO\r\n", "Go\r\n", "GO \r\n", "Go \r\n", "go\r\n", "\r\ngo", "Go \r\n" };
                string[] arrSQL = script.Split(_delimeter, StringSplitOptions.RemoveEmptyEntries);


                int totalScripts = arrSQL.Length;
                int scriptCounterStep = Convert.ToInt32(100 / totalScripts) + 1;
                int scriptCounter = 0;

                OnProgress(this, new ProgressEventArgs((scriptCounter += scriptCounterStep)));

                SqlCommand cmd = new SqlCommand();
                cmd.Connection = con;
                cmd.CommandTimeout = 0;
                cmd.CommandType = System.Data.CommandType.Text;

                if (bTran)
                {
                    SqlTransaction tran = con.BeginTransaction();
                    cmd.Transaction = tran;
                    try
                    {
                        foreach (string sql in arrSQL)
                        {
                            if (this.SetupGridUC.HaltSetup)
                                return false;
                            OnProgress(this, new ProgressEventArgs((scriptCounter += scriptCounterStep)));
                            if (sql.Trim() == string.Empty)
                                continue;
                            cmd.CommandText = sql;
                            cmd.ExecuteNonQuery();
                        }

                        tran.Commit();
                    }
                    catch (SqlException ex)
                    {
                        tran.Rollback();
                        //SA.Common.WinUI.Utility.HandleException(ex, "بروز خطا در به روز کردن پایگاه داده", typeof(SA.Sys.Model.AppData).FullName, "Upgrade", null);
                        throw ex;
                    }
                }
                else
                {
                    foreach (string sql in arrSQL)
                    {
                        OnProgress(this, new ProgressEventArgs((scriptCounter += scriptCounterStep)));
                        try
                        {
                            if (this.SetupGridUC.HaltSetup)
                                return false;
                            if (sql.Trim() == string.Empty)
                                continue;
                            cmd.CommandText = sql;
                            cmd.ExecuteNonQuery();
                        }
                        catch (SqlException ex)
                        {
                            //SA.Common.WinUI.Utility.HandleException(ex, "بروز خطا در به روز کردن پایگاه داده", typeof(SA.Sys.Model.AppData).FullName, "Upgrade", null);
                            bret = false;
                            throw ex;
                        }
                    }
                }

                if (con.State == ConnectionState.Open)
                    con.Close();
            }
            /**/
            return bret;
        } 
        #endregion
    }
}
